#========================================================================================
# package      DE_EPAGES::Customer::API::User
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides user functions.
#========================================================================================
package DE_EPAGES::Customer::API::User;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    UpdateCustomerTaxModel
);

use DE_EPAGES::Customer::API::Shop  qw ( GetCustomerForUser );
use DE_EPAGES::Shop::API::Address   qw ( IsTaxModelNet );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET TAX_MODEL_GROSS );

#========================================================================================
# function     UpdateCustomerTaxModel
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateCustomerTaxModel($User, $Shop);
#----------------------------------------------------------------------------------------
# description  Updates the tax model of the customer based on the user billing address.
#               Set tax model to net, when country of billing address in eu and vatid
#               is valid.
#               Set the container TaxModel to gross is shop is VAT released.
#----------------------------------------------------------------------------------------
# input        $User | user | object
# input        $Shop | shop | object
# return       $NewTaxModel | new tax model if tax model was changed, otherwise undef | int
#========================================================================================
sub UpdateCustomerTaxModel {
    my ($User, $Shop) = @_;
    my $Customer = GetCustomerForUser($User, $Shop);

    return unless $Customer;

    # test if we have a VAT released shop
    # set the TaxModel to gross  in every case
    if ($Shop->get('VATreleased')) {
        $Customer->set({'TaxModel' => TAX_MODEL_GROSS});
        return TAX_MODEL_GROSS;
    }

    # set the tax model by testing the BillingAddress
    # we must have a billing address in every case
    my $hUserAddresses = $User->get(['BillingAddress']);
    return unless $hUserAddresses->{'BillingAddress'};

    my $OldTaxModel = $Customer->get('TaxModel');

    # test the billing address and switch to net if needed
    if (IsTaxModelNet($Shop, $hUserAddresses->{'BillingAddress'})) {
        $Customer->set({'TaxModel' => TAX_MODEL_NET}) unless $OldTaxModel == TAX_MODEL_NET;
        return TAX_MODEL_NET;
    }

    return;
}

1;
