#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete GoogleSmartShopping related objects.
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::Product::API::Object::LineItemProduct qw ( GetLineItemProductAsHash );
use DE_EPAGES::GoogleSmartShopping::API::DialogBuilder;


use strict;

#========================================================================================
# function     JSONSearchCampaignProductsMetrics
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONSearchCampaignProducts($Servlet);
#----------------------------------------------------------------------------------------
# description  JSON handler for products campaign metrics search
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub JSONSearchCampaignProductsMetrics {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hFormValues = $Servlet->form()->form( $Shop, 'JSONSearchCampaignProductsMetrics' );
    my $LanguageID = $Servlet->languageID();

    # Trigger search.
    my $DialogBuilder = DE_EPAGES::GoogleSmartShopping::API::DialogBuilder->new(
        {
            'Shop'       => $Shop,
            'FormValues' => $hFormValues,
            'LanguageID' => $LanguageID
        }
    );
    my $hProductsCampaignPage = $DialogBuilder->fetchCampaignProductsPage();
    my $hDialogData = $DialogBuilder->buildCampaignProductsDialog($hProductsCampaignPage);

    # write result data on the current servlet.
    $self->writeContentOutput($hDialogData);
}

sub JSONGetDataForGSS {
    my $self = shift;
    my ($Servlet) = @_;

    my $ObjectGuids = $Servlet->form->values->{'IDs[]'};
    my $Type = $Servlet->form->values->{'type'};

    my $LineItem;
    my $aLineItemProducts = [];
    my $hResult;
    if (@$Type[0] eq 'order') {

        foreach my $GUID ( @$ObjectGuids ) {
            my $Order = LoadObjectByGUID($GUID);

            my $LineItemContainer = $Order->child('LineItemContainer');
            my $aChildObjects = $LineItemContainer->childrenByClass('LineItemProduct');

            my $aProductLineItems = [];
            foreach my $LineItemPrd (@$aChildObjects) {
                push (@$aProductLineItems, $LineItemPrd->GetLineItemProductAsHash($Servlet, $Type));
            }

            $hResult = {
                'lineItemContainer' => {
                    'grandTotal'       => $self->_getGrandTotal($LineItemContainer),
                    'productLineItems' => $aProductLineItems,
                },
                # check if alias is the right one
                'orderNumber'       => $Order->alias,
                'success' => JSON::true,
            };
        }
    }
    if (@$Type[0] eq 'cart') {
        foreach my $GUID ( @$ObjectGuids ) {
            $LineItem = LoadObjectByGUID($GUID);
            push @$aLineItemProducts, $LineItem->GetLineItemProductAsHash($Servlet, $Type);
        }

        $hResult->{'productLineItems'} = $aLineItemProducts;

        my $LineItemContainer = $LineItem->get('LineItemContainer');
        $hResult->{'grandTotal'} = $self->_getGrandTotal($LineItemContainer);
        $hResult->{'success'} = JSON::true;
    }

    return $self->writeContentOutput($hResult);
}

sub _getGrandTotal {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $Shop = $LineItemContainer->getSite();

    my $Amount = $LineItemContainer->get('GrandTotal');
    my $hContainerInfo = $LineItemContainer->get([ qw( CurrencyID TaxModel LocaleID ) ]);

    my $Formatter = $Shop->siteLocale($hContainerInfo->{'LocaleID'});
    my $Formatted = $Formatter->format( $Amount, 'money', $hContainerInfo->{'CurrencyID'} );

    return {
        'taxType'    => $hContainerInfo->{'TaxModel'} ? 'GROSS' : 'NET',
        'amount'     => $Amount * 1,
        'currency'   => $hContainerInfo->{CurrencyID},
        'formatted'  => $Formatted,
    };
}

1;
