#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping ProductFeed tab
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::GoogleSmartShopping::UI::ProductFeed;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object String HashRef ArrayRef Number );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use HTTP::Status qw (HTTP_OK );

use Test::More;

use strict;

use constant MERCHANT_CENTER_ID => 503220055;
use constant GOOGLE_PORTAL_ALIAS => 'SomeAlias_de_DE';
use constant DEFAULT_SHIPPING_METHOD_ID => '22528';
use constant DEFAULT_SHIPPING_METHOD_ALIAS => 'Express';
use constant PORTAL_CONFIG_ID => 78910;
use constant PORTAL_SITE_LANGUAGE_CODE => 'ger';

sub test {
    my $self = shift;

    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics();
    $self->test_ChangePortalShippingMethod_Success_without_Products();
    $self->test_ChangePortalShippingMethod_Success_with_Products();
    $self->test_ChangePortalShippingMethod_fail_no_PortalConfig();
    $self->test_UploadAllProducts();
    $self->test_ExportPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Show_MerchantCenterProductsTable();
    $self->test_RemoveProducts();
    $self->test_RemoveProducts_upload_all_products_set();
    $self->test_RemoveAllProducts_success();
    $self->test_ReuploadProducts();
    done_testing();
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
         MockedAddress      => $MockedAddress,
         MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
        ID         => PORTAL_CONFIG_ID,
        LanguageID => 1
    });

    my $hPostData = {};
    my $ServletForm = $self->_createServletForm($hPostData);

    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $ServletForm);
    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUT(
        {'GetInfoLanguage' => $hReturnFromInfoLanguage }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
    is ($MockedServlet->vars('PortalConfigID'), PORTAL_CONFIG_ID, $SubTestName . ' - Verify that PortalConfigID is set.' );
    is ($MockedServlet->vars('PortalSiteLanguageCode3'), PORTAL_SITE_LANGUAGE_CODE, $SubTestName . ' - Verify that PortalSiteLanguageCode3 is set.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUT(
        {'GetInfoLanguage' => $hReturnFromInfoLanguage }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = undef;
    my $MockedAddress = $self->_createAddress($hCountry);
    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUT(
        {'GetInfoLanguage' => $hReturnFromInfoLanguage }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );

}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUT(
        {'GetInfoLanguage' => $hReturnFromInfoLanguage }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );
    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 0,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUT(
        {'GetInfoLanguage' => $hReturnFromInfoLanguage }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0,
        total               => 5,
        active              => 5,
        pending             => 0,
        disapproved         => 0
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });
    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $hParams = {
        'GoogleApiService'          => $GoogleApiService,
        'GetInfoLanguage' => $hReturnFromInfoLanguage
    };

    my $SUT = $self->_createSUT($hParams);

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok(WasCalled($GoogleApiService, 'getRatedProductsStatistics'),
        $SubTestName . ' - Verify that service method _getProductStatistics was called.');
    is_deeply(
        $MockedServlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}

sub test_ChangePortalShippingMethod_Success_without_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
        ShippingMethod => DEFAULT_SHIPPING_METHOD_ID
    });

    my $ShippingMethodId = '8198228';
    my $hFormValues = {
        selectedShippingMethodId => [$ShippingMethodId],
    };
    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        $ShippingMethodId,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with updated ShippingMethodID'
    );
}

sub test_ChangePortalShippingMethod_Success_with_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
        Products => ['12334', '87101'],
        ShippingMethod => _createShippingMethod(),
    });
    my $ShippingMethodId = '8198228';
    my $hFormValues = {
        selectedShippingMethodId => [$ShippingMethodId],
    };

    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        $ShippingMethodId,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the unchanged ShippingMethod'
    );
}

sub test_ChangePortalShippingMethod_fail_no_PortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $hFormValues = {};
    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        undef,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called without ShippingMethod, when no PortalConfig'
    );
}

sub test_UploadAllProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        ExportAllProducts => 0,
        MockedShop => $MockedShop,
    });
    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->UploadAllProducts($MockedServlet);

    my $hParams = GetParametersFromMockifyCall($MockedPortalConfig, 'set')->[0];

    # actual tests
    is_deeply(
        $hParams,
            { 'ExportAllProducts' => 1 },
        $SubTestName . ' - Ensure that the ExportAllProducts boolean was set to true'
    );

    ok( WasCalled($SUT, 'DE_EPAGES::ProductPortal::UI::PortalConfig::ExportPortalConfig'), $SubTestName . ' - Verify that SUT "ExportPortalConfig" method was called.' );
}

sub test_ExportPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({ });

    my $MockedPortalConfig = $self->_createPortalConfig({
        ExportAllProducts => 1,
        MockedShop => $MockedShop,
    });
    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->ExportPortalConfig($MockedServlet);

    my $hParams = GetParametersFromMockifyCall($MockedPortalConfig, 'set')->[0];

    # actual tests
    is_deeply(
        $hParams,
        { 'ExportAllProducts' => 0 },
        $SubTestName . ' - Ensure that the ExportAllProducts boolean was set to false'
    );

    ok( WasCalled($SUT, 'DE_EPAGES::ProductPortal::UI::PortalConfig::ExportPortalConfig'), $SubTestName . ' - Verify that SUT "ExportPortalConfig" method was called.' );
}


sub test_UnityViewGoogleSmartShoppingProductFeed_Show_MerchantCenterProductsTable {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0,
        total               => 6,
        active              => 4,
        pending             => 0,
        disapproved         => 2
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $PagerMerchantCenterProducts = $self->_createPagerMerchantCenterProducts();

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $hParams = {
        'GoogleApiService'            => $GoogleApiService,
        'PagerMerchantCenterProducts' => $PagerMerchantCenterProducts,
        'SpyViewPagedChildren'        => 1,
        'GetInfoLanguage' => $hReturnFromInfoLanguage,
    };

    my $SUT = $self->_createSUT($hParams);

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok(WasCalled($SUT, '_viewPagedChildren'),
        $SubTestName . ' - Verify that _viewPagedChildren was called.');

    ok(WasCalled($PagerMerchantCenterProducts, 'execute'),
        $SubTestName . ' - Verify that pager execute was called.');


    ok(WasCalled($PagerMerchantCenterProducts, 'page'),
        $SubTestName . ' - Verify that pager page was called.');

    is_deeply(
        $MockedServlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}


sub test_RemoveProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
    });

    my $hPostData = {'ListObjectID' =>  ['online:de:45-hat_de_DE', 'online:de:32-bracelet_de_DE' ] };
    my $hServletForm = $self->_createServletForm($hPostData);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $hServletForm, $MockedServletResponse);
    my $MockedGoogleApiService = $self->_createGoogleApiService();
    my $MockedGooglePortalService = $self->_createGooglePortalService({'Shop' => $MockedShop, 'aProductGoogleIDs' => [],'aProductAliases' =>[]});
    my $hSutParams = {
        GoogleApiService    => $MockedGoogleApiService,
        GooglePortalService => $MockedGooglePortalService
    };
    my $SUT = $self->_createSUT($hSutParams);

    $SUT->RemoveProducts($MockedServlet);


    ok(
        WasCalled($SUT, '_removeProductsInMerchantCenter'),
        "$SubTestName . - Ensure that _removeProductsInMerchantCenter was called correctly"
    );

    ok(
        WasCalled($MockedGooglePortalService, 'getProductAliasByGoogleID'),
        "$SubTestName . - Ensure that _removeProductsInMerchantCenter was called correctly"
    );

    ok(
        WasCalled($MockedGooglePortalService, 'deleteProductsInPortalConfig'),
        "$SubTestName . - Ensure that _removeProductsInMerchantCenter was called correctly"
    );


    ok(
        WasCalled($SUT, 'DE_EPAGES::GoogleSmartShopping::API::GooglePortalService::new'),
        "$SubTestName . - Ensure that GooglePortalService was called correctly"
    );
}

sub test_RemoveProducts_upload_all_products_set {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop        => $MockedShop,
        ExportAllProducts => 1
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

        $SUT->RemoveProducts($MockedServlet);

    ok(
        !WasCalled($SUT, '_removeProductsInMerchantCenter'),
        "$SubTestName . - Ensure that _removeProductsInMerchantCenter was not called."
    );

    ok(
        !WasCalled($SUT, '_removeProductsInPortal'),
        "$SubTestName . - Ensure that _removeProductsInMerchantCenter was not called."
    )
}

sub test_RemoveAllProducts_success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop({});
    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
    });
    my $MockedServlet = $self->_createServlet($MockedPortalConfig);
    my ($aExpectedProducts, $aExpectedProductsStatusesResponse) = $self->_generateExpectedProductsStatusesResponse();
    my $GoogleApiService = $self->_createGoogleApiService({
        ProductsStatuses => $aExpectedProductsStatusesResponse,
    });
    my $ProductBatchService = $self->_createProductBatchService();
    my $hParams = {
        'GoogleApiService'    => $GoogleApiService,
        'ProductBatchService' => $ProductBatchService
    };

    my $SUT = $self->_createSUT($hParams);

    $SUT->RemoveAllProducts($MockedServlet);

    ok(
        WasCalled($GoogleApiService, 'productsStatuses'),
        $SubTestName . ' - Verify that service method productsStatuses was called.'
    );
    ok(
        WasCalled($ProductBatchService, 'OnStartBatch'),
        $SubTestName . ' - Verify that service initialization method OnStartBatch was called.'
    );
    ok(
        WasCalled($ProductBatchService, 'deleteProducts'),
        $SubTestName . ' - Verify that service method deleteProducts was called.'
    );
    my $aProductsCall1 = GetParametersFromMockifyCall($ProductBatchService, 'deleteProducts', 0)->[0];
    my @ExpectedProductsCall1 = splice(
        @$aExpectedProducts, 0, DE_EPAGES::GoogleSmartShopping::UI::ProductFeed::AMOUNT_OF_PRODUCTS_PER_BATCH
    );
    is_deeply(
        $aProductsCall1,
        \@ExpectedProductsCall1,
        $SubTestName . ' - Verify that arguments of deleteProducts for the first call are correct.'
    );
    my $aProductsCall2 = GetParametersFromMockifyCall($ProductBatchService, 'deleteProducts', 1)->[0];
    my @ExpectedProductsCall2 = splice(
        @$aExpectedProducts, 0, DE_EPAGES::GoogleSmartShopping::UI::ProductFeed::AMOUNT_OF_PRODUCTS_PER_BATCH
    );
    is_deeply(
        $aProductsCall2,
        \@ExpectedProductsCall2,
        $SubTestName . ' - Verify that arguments of deleteProducts for the second call are correct.'
    );
}

sub test_ReuploadProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop({ });

    my $MockedPortalConfig = $self->_createPortalConfig({
        ExportAllProducts => 0,
        MockedShop => $MockedShop,
    });
    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();
    $SUT->ReUploadAllProducts($MockedServlet);

    ok(WasCalled($SUT, 'DE_EPAGES::ProductPortal::UI::PortalConfig::GeneratePortalConfigProductPrettyUnityUrls'), $SubTestName . ' - Verify SUT Super "GeneratePortalConfigProductPrettyUnityUrls" was called');

    ok( WasCalled($SUT, 'DE_EPAGES::ProductPortal::UI::PortalConfig::ExportPortalConfig'), $SubTestName . ' - Verify that SUT Super "ExportPortalConfig" method was called.' );
}


#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::ProductFeed');

    $MockifySut->spy('_areProductUploadRequirementsMet')->when(Object());
    $MockifySut->spy('_removeProductsInMerchantCenter')->when(Object(), ArrayRef());
    $MockifySut->spy('_removeProductsInPortal')->when(Object(), Object(), ArrayRef());

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->mockStatic('DE_EPAGES::ProductPortal::UI::PortalConfig::ExportPortalConfig')
        ->when(Object(), Object(), Number(0))
        ->thenReturnUndef();

    $MockifySut->mockStatic('DE_EPAGES::ProductPortal::UI::PortalConfig::GeneratePortalConfigProductPrettyUnityUrls')
        ->when(Object(), Object())
        ->thenReturnUndef();

    $MockifySut->mockStatic('DE_EPAGES::ProductPortal::UI::PortalConfig::RemoveProducts')
        ->when(Object())
        ->thenReturnUndef();

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GoogleApiService'});
    }
    if (defined $hParams->{'ProductBatchService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::ProductBatchService::new')
            ->when(String(), Object())
            ->thenReturn($hParams->{'ProductBatchService'});
    }
    if (defined $hParams->{'PagerMerchantCenterProducts'}) {
    $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts::new')
        ->when(String(), HashRef())
        ->thenReturn($hParams->{'PagerMerchantCenterProducts'});
}
    if (defined $hParams->{'GooglePortalService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GooglePortalService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GooglePortalService'});
    }

    if (defined $hParams->{'GetInfoLanguage'}) {
        $MockifySut->mockImported('DE_EPAGES::Object::API::Table::Language', 'GetInfoLanguage')
            ->when(Number())
            ->thenReturn($hParams->{'GetInfoLanguage'});
    }

    if (defined($hParams->{'NewGooglePortalService'})) {
        $MockifySut->mock('_createPortalConfig')
            ->whenAny()
            ->thenReturn($hParams->{'NewGooglePortalService'});
    }

    if(defined($hParams->{'SpyViewPagedChildren'})) {
        $MockifySut->spy('_viewPagedChildren')->when(Object());
    } else {
        $MockifySut->mock('_viewPagedChildren')
            ->whenAny()
            ->thenReturnUndef();
    }

    return $MockifySut->getMockObject();
}

sub _createPortalConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'ExportAllProducts' => $hParams->{'ExportAllProducts'} // undef,
        'ShippingMethod'    => $self->_createShippingMethod(),
        'NameOrAlias'       => 'NameOrAlias',
        'PortalConfigClass' => 'GoogleSmartShopping',
        'ID'                => $hParams->{'ID'} // '12345',
        'Site'              => $hParams->{'MockedShop'} // {},
        'Products'          => $hParams->{'Products'} // [],
        'LanguageID'        => $hParams->{'LanguageID'} // undef
    };

    my $MockifyPortal = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig', $hAttributes
    );
    $MockifyPortal->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);
    $MockifyPortal->addMockWithReturnValue('removeAllProducts', undef);
    $MockifyPortal->addMockWithReturnValueAndParameterCheck('removeProducts', undef, [ 'arrayref' ]);

    return $MockifyPortal->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'Address'                   => $hParams->{'MockedAddress'} // undef,
        'GoogleSmartShoppingConfig' => $hParams->{'MockedGoogleConfig'} // undef,
        'IsUnityRegistered'         => 1
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('existsChild', defined($hParams->{'MockedGoogleConfig'}), [{'string'=>'GoogleSmartShoppingConfig'}]);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('folder', $self->_createFolderObject, [{'string'=>'Products'}]);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('loadObject',$self->_createShippingMethod(), ['string', 'string']);

    return $MockifyShop->getMockObject();
}

sub _createGoogleConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC                => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo               => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies         => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID            => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID                 => $hConfig->{'GoogleAdsAccountID'} // undef,
        AdsCampaignId                      => $hConfig->{'AdsCampaignId'} // undef,
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'} // undef,
        AreMerchantCenterAndAdsLinked      => $hConfig->{'AreMerchantCenterAndAdsLinked'} // 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($PortalConfig, $ServletForm, $ServletResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($PortalConfig);
    if (defined $ServletForm && defined $ServletResponse) {
        $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
        $MockifyServlet->addMockWithReturnValue('response', $ServletResponse);
        $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, [ 'hashref' ]);
        $MockifyServlet->addMockWithReturnValueAndParameterCheck('params', undef, [ 'string', 'string' ]);
    }


    return $MockifyServlet->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hPostData) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');

    if( defined $hPostData->{'ListObjectID'}) {
        $Mockify->mock('value')
            ->when(String('ListObjectID'))
            ->thenReturn($hPostData->{'ListObjectID'});
    }

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hPostData);

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenThrowError('Something bad happened! fail;<');

    return $Mockify->getMockObject();
}

sub _createServletResponse {
    my $self = shift;

    my $Mockify = Test::Mockify->new('HTTP::Response');
    $Mockify->mock('code')
        ->whenAny()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hCountry) = @_;

    my $hParameterList = {
        'Country'   => $hCountry ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('getRatedProductsStatistics')
        ->when()
        ->thenReturn($hServiceResponse->{'RatedProductsStatistics'} // {}
    );

    $Mockify->mock('deleteProducts')
        ->when(ArrayRef())
        ->thenReturnUndef();

    $Mockify->mock('productsStatuses')
        ->when(Number(), Number())
        ->thenReturn($hServiceResponse->{'ProductsStatuses'} // {}
    );

    return $Mockify->getMockObject();
}

sub _createPagerMerchantCenterProducts {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts');

    $Mockify->mock('execute')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('page')
        ->when()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createShippingMethod {
    my $self = shift;
    my $hParameterList = {
        'ID'          => DEFAULT_SHIPPING_METHOD_ID,
        'NameOrAlias' => DEFAULT_SHIPPING_METHOD_ALIAS
    };
    my $MockifyShippingMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ShippingTypes::API::Object::ShippingMethodWeightOfBasket',
        $hParameterList
    );

    return $MockifyShippingMethod->getMockObject();
}

sub _createFolderObject {
    my $self = shift;

    my $Mockify =  Test::Mockify->new('DE_EPAGES::Object::API::Object::Object');

    $Mockify->mock('existsChild')
        ->when(String())
        ->thenReturn(1);

    $Mockify->mock('child')
        ->when(String())
        ->thenReturn($self->_createProduct());

    return $Mockify->getMockObject();
}

sub _createProduct {
    my $self = shift;
    my $hParameterList = {
        Alias => '32-Bica',
        ID    => 12345,
    };

    my $MockProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product',
        $hParameterList
    );

    return $MockProduct->getMockObject();
}

sub _createGooglePortalService {
    my $self = shift;
    my ($hParams) = @_;

    my $hParameterList = {
        Shop => $hParams - { 'Shop' },
    };

    my $MockPortalService = Test::Mockify->new(
        'DE_EPAGES::GoogleSmartShopping::API::GooglePortalService', [ $hParameterList ]);

    $MockPortalService->mock('deleteProductsInPortalConfig')
        ->when(ArrayRef())
        ->thenReturnUndef();

    if (defined $hParams->{'aProductGoogleIDs'}) {
        $MockPortalService->mock('getProductAliasByGoogleID')
            ->when(ArrayRef())
            ->thenReturn($hParams->{'aProductAliases'});
    }

    return $MockPortalService->getMockObject();
}

sub _createProductBatchService {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::ProductBatchService');

    $Mockify->mock('OnStartBatch')
        ->when()->thenReturnUndef();
    $Mockify->mock('deleteProducts')
        ->when(ArrayRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _generateExpectedProductsStatusesResponse {
    my $self = shift;

    my @Products = ();
    my @StatusesResponse = ();
    my $NumberOfProducts = DE_EPAGES::GoogleSmartShopping::UI::ProductFeed::AMOUNT_OF_PRODUCTS_PER_BATCH + 5;
    my $Counter = 0;
    while ($Counter < $NumberOfProducts) {
        $Counter++;
        my $ProductId = 'online:de:DE:' . $Counter . '_de_DE';
        push(@Products, $ProductId);
        push(@StatusesResponse, { productId => $ProductId });
    }
    return ( \@Products, { items => \@StatusesResponse } );
}

__PACKAGE__->runScript();
