#========================================================================================
# package      DE_EPAGES::MetaCatalog::API::Object::MetaCatalogPortalConfig
# base         DE_EPAGES::ProductPortal::API::Object::PortalConfig
# state        public
#----------------------------------------------------------------------------------------
# description  object interface for PortalConfig.
#========================================================================================
package DE_EPAGES::MetaCatalog::API::Object::MetaCatalogPortalConfig;
use base DE_EPAGES::ProductPortal::API::Object::PortalConfig;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::MetaCatalog::API::Object::MetaCatalogPortalConfig->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $ProductClass = LoadClassByAlias('Product');
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'ImageAttribute' => $ProductClass->attribute('ImageLarge'),
        'ExportInterval' => 1, # days automatic export launches since last export
        'ExportAllProducts' => 1,
        'ExportAllVariations' => 1,
    };
}

1;
