#========================================================================================
# package      DE_EPAGES::MetaCatalog::API::TLE::MetaCatalogHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This class contains common methods for TLE functions.
#========================================================================================
package DE_EPAGES::MetaCatalog::API::TLE::MetaCatalogHandler;

use strict;
use DE_EPAGES::MetaCatalog::API::Tools qw ( FindMetaCatalogPortalConfig );

#---------------------------------------------------------------------------
# TLE Registration
#---------------------------------------------------------------------------
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'}, "GetMetaCatalogPortalConfig" );

    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# input        %args | args | array
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;

    return $self;
}

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor, $TLEFunctionName );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor       | tle processor     | object
# input        $TLEFunctionName | TLE Function Name | string
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor, $TLEFunctionName) = @_;

    $Processor->registerHandler( 'FunctionHandler', $self, $TLEFunctionName );

    return;
}

#========================================================================================
# function     GetMetaCatalogPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hInfo = GetMetaCatalogPortalConfig( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# description  Return the portal config object for Meta Catalog if found
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       $hTLEHash | TLE Hash of the portal config object | ref.hash.object
#========================================================================================
sub GetMetaCatalogPortalConfig {
    my $self                 = shift;
    my ($Processor,$aParams) = @_;

    my $Object = $aParams->[0];
    my $Shop = $Object->getSite();

    my $MetaPortalConfig = FindMetaCatalogPortalConfig($Shop);
    return $MetaPortalConfig ? $MetaPortalConfig->tleHash : undef;
}


1;
