#==============================================================================
# §package      DE_EPAGES::MetaCatalog::API::Tools
# §base         Exporter
# §state        public
#------------------------------------------------------------------------------
# §description  Helper subroutines
#==============================================================================
package DE_EPAGES::MetaCatalog::API::Tools;
use base Exporter;

use strict;
use DE_EPAGES::MetaCatalog::API::Constants qw ( $GLOBAL_PORTAL_CONFIG_ALIAS );

our @EXPORT_OK = qw (
  FindMetaCatalogPortalConfig
);

#========================================================================================
# §function     FindMetaCatalogPortalConfig
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hInfo = FindMetaCatalogPortalConfig( $Shop );
#----------------------------------------------------------------------------------------
# §description  Find the meta catalog portal config object of this shop
#----------------------------------------------------------------------------------------
# §input        $Shop | Shop class object | object
# §return       $PortalConfig | Portal Config object | object
#========================================================================================
sub FindMetaCatalogPortalConfig {
    my ($Shop) = @_;

    my $aPortalConfigs = $Shop->get('PortalConfigs');
    return undef unless defined $aPortalConfigs;

    foreach my $PortalConfig (@{$aPortalConfigs}){
        if (index($PortalConfig->get('Alias') ,'MetaCatalog') != -1) {
            if ( $PortalConfig->get('PortalSite') && $PortalConfig->get('PortalSite')->alias eq $GLOBAL_PORTAL_CONFIG_ALIAS ) {
                return $PortalConfig;
            }
        }
    }

    return undef;
}

1;
