#==============================================================================
# §package      DE_EPAGES::MetaCatalog::UI::Shop
# §base         DE_EPAGES::Presentation::UI::Object
# §state        private
#------------------------------------------------------------------------------
# §description  Shop UI subroutines
#==============================================================================
package DE_EPAGES::MetaCatalog::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias );
use DE_EPAGES::ProductPortal::API::Object::PortalConfig;
use DE_EPAGES::Product::API::Object::PagerProducts;
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Search::API::Object::PagerSearch;

use DE_EPAGES::MetaCatalog::API::Tools qw ( FindMetaCatalogPortalConfig );
use DE_EPAGES::MetaCatalog::API::Constants qw ( $GLOBAL_PORTAL_CONFIG_ALIAS );

#========================================================================================
# §function     MetaCatalogViewGeneral
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::MetaCatalog::UI::Shop->MetaCatalogViewGeneral($Servlet);
#----------------------------------------------------------------------------------------
# §description  UI subroutine for the general page of the Meta Catalog integration
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet object | object
#========================================================================================
sub MetaCatalogViewGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $MetaPortalConfig = FindMetaCatalogPortalConfig($Shop);
    my $PortalSite = $MetaPortalConfig ? $MetaPortalConfig->get('PortalSite') : undef;

    if ( ! $MetaPortalConfig ) {
        my $System = LoadRootObject();
        my $aPortals = $System->get('Portals');

        foreach my $Portal (@{$aPortals}) {
            if ($Portal->get('NameOrAlias') eq 'MetaCatalog') {
                my $Class = $Portal->get('PortalConfigClass');

                #-- Insert the 'default' portal. Remember, we insert this irrespective of country/language.
                foreach my $LocalizedPortalSite (@{$Portal->get('PortalSites')}) {
                    if ($LocalizedPortalSite->alias eq $GLOBAL_PORTAL_CONFIG_ALIAS) {
                        $PortalSite = $LocalizedPortalSite;
                        
                        $MetaPortalConfig = $self->NewObject($Servlet, $Shop->folder('PortalConfigs'), $Class, {
                            'Alias'      => $PortalSite->alias,
                            'PortalSite' => $PortalSite,
                        });
                    }
                }
            }
        }
    }

    $Servlet->vars('MetaCatalogPortalConfig', $MetaPortalConfig->tleHash);
    my $ExportViaPixel = $MetaPortalConfig->get('ExportViaPixel');

    my $UnityScriptTagsAppClient = GetUnityScriptTagsApp($Shop);
    my $ScriptUrl = $Shop->get('WebServerScriptName') . '?ViewAction=UnityViewMetaCatalogScript';
    if ( $ExportViaPixel ) {
        $UnityScriptTagsAppClient->addScriptTag($ScriptUrl);
    } else {
        $UnityScriptTagsAppClient->deleteScriptTag($ScriptUrl);
    }

    return $Shop;
}


#========================================================================================
# §function     MetaCatalogViewProducts
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::MetaCatalog::UI::Shop->MetaCatalogViewProducts($Servlet);
#----------------------------------------------------------------------------------------
# §description  UI subroutine for the products page of the Meta Catalog integration
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet object | object
#========================================================================================
sub MetaCatalogViewProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $MetaPortalConfig = FindMetaCatalogPortalConfig($Shop);

    $Servlet->vars('MetaCatalogPortalConfig', $MetaPortalConfig->tleHash);

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('Product'), 'MBO-PortalConfig');
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $MetaPortalConfig,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'PortalConfig.ID'  => $MetaPortalConfig->id,
            'SearchLanguageID' => $Servlet->languageID,
        },
        'UriParameters'  => {},
    );

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

1;