#==============================================================================
# package      DE_EPAGES::Ogone::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object;
# state        private
#==============================================================================
package DE_EPAGES::Ogone::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::ExternalPayment::API::Log qw (LogPayment);
use DE_EPAGES::Ogone::API::Constants qw (@CREDITCARDS @SPLIT_GATEWAY_METHODS);
use DE_EPAGES::Ogone::API::Tools qw ( Ogone_SHA1 GetAddressEmail );
use DE_EPAGES::Object::API::Factory qw (LoadObjectByGUID LoadRootObject);

use constant PERCENT_FACTOR => 100 ;

#==============================================================================
# Event handlers
#==============================================================================
sub ViewPaymentPostfinance {
    my $self = shift;
    my ($Servlet) = @_;
    $self->ViewPaymentOgone($Servlet);
    return;
}

sub ViewPaymentAcceptance {
    my $self = shift;
    my ($Servlet) = @_;
    $self->ViewPaymentOgone($Servlet);
    return;
}

sub ViewPaymentOgone {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Object  = $Servlet->object;    # may be a basket or a customerorder


    my $Container = $Object->container;
    return unless defined $Container;

    my $LineItemPayment = $Object->get('LineItemContainer')->get('Payment');
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');
    my $LineItemPaymentPID = $LineItemPayment->get('ID');
    my $Basket = $LineItemPayment->container->parent;

    my $Shop = $Object->getSite;
    my $passphrase = $PaymentMethod->get('IsInTestMode') ?
                        $PaymentMethod->get('PassphraseTest') :
                        $PaymentMethod->get('Passphrase');

    # Here we build the request that we send to Ogone.
    # if the selected payment method is acredit card, we send
    # PM = 'CreditCard' and BRAND = NamePaymentMethod
    # Otherwise the field BRAND is empty

    my $SplitGatewayMethod = $PaymentMethod->get('SplitGatewayMethod');
    my $Key = 'REAL_SGM';

    my %hSPLIT_GATEWAY_METHODS = map { ($_->{'ID'}) => $_ } @SPLIT_GATEWAY_METHODS;
    my $nameMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'Name'};
    my $brandMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'Brand'};
    my $paymentMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'PaymentMethod'};

    # payment parameters
    my %Params = (
        'PSPID'         => $PaymentMethod->get('IsInTestMode') ?
                                $PaymentMethod->get('MerchantNameTest') :
                                $PaymentMethod->get('MerchantName'),
        'ORDERID'       => sprintf('%s_%s', $Object->id, GetCurrentDateTime()),
        'AMOUNT'        => $Object->get('LineItemContainer')->get('GrandTotal') * PERCENT_FACTOR,
        'CURRENCY'      => $LineItemPayment->get('CurrencyID'),
        'LANGUAGE'      => $Object->get('LineItemContainer')->get('LocaleID'),
        'PM'            => (grep {$_->{$Key} eq $SplitGatewayMethod} @CREDITCARDS) ?
                                'CreditCard' :
                                $paymentMethod,
        'BRAND'         => (grep {$_->{$Key} eq $SplitGatewayMethod} @CREDITCARDS) ?
                                $nameMethod :
                                $brandMethod,
        'ORIG'          => 'ePages',
    );

    my $ChangeActionOnAccept = 'PaymentBacklinkOgone' ;

    if ( $nameMethod eq 'Amazon Checkout' ) {

        # The selected payment method is Amazon Checkout !
        # We use a different Action in the ACCEPT URL's (!)

        $ChangeActionOnAccept = 'PaymentBacklinkOgoneAmazon' ;
    }

    $Params{'ACCEPTURL'}  =
        BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => $ChangeActionOnAccept,
                'PID' => $LineItemPaymentPID,
                'RET' => 'SUCCESS'
            },
            {
                'UseSSL' => 1,
                'UseObjectPath'=> 0
            }
        ) ;

    $Params{'DECLINEURL'} =
        BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID' => $LineItemPaymentPID,
                'RET' => 'DECLINED'
            },
            {
                'UseSSL' => 1,
                'UseObjectPath'=> 0
            }
        ) ;

    $Params{'EXCEPTIONURL'} =
        BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID' => $LineItemPaymentPID,
                'RET' => 'EXCEPTION'
            },
            {
                'UseSSL' => 1,
                'UseObjectPath'=> 0
            }
        ) ;

    $Params{'CANCELURL'} =
        BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID' => $LineItemPaymentPID,
                'RET' => 'CANCELED'
            },
            {
                'UseSSL' => 1,
                'UseObjectPath'=> 0
            }
        ) ;

    $Params{'BACKURL'} =  $Basket->getSite()->get('WebUrl');

    $Params{'HOMEURL'} =  $Basket->getSite()->get('WebUrl');

    my $paymentTypeName = $LineItemPayment->get('PaymentType')->get('Alias');
    if ($paymentTypeName eq 'Postfinance') {
        # because Postfinace attribute names (PostFinance_...) don't match with payment type (Postfinance)
        # we check it and change if required. The most elegant solution would be to make match both at DB/XML level.
        $paymentTypeName = 'PostFinance';
        # Postfinance-TWINT requires to send the EMail
        if ($SplitGatewayMethod eq 'TWINT') {
            $Params{'EMAIL'} = GetAddressEmail($Basket->get('BillingAddress'));
        }

    }

    # calculate SHASIGN
    $Params{'SHASIGN'} = Ogone_SHA1($passphrase, \%Params);

    # set status and transid
    my $isTestMode = $PaymentMethod->get('IsInTestMode');
    $LineItemPayment->set( {
            'IsTestTransaction' => $isTestMode,
            'TransID' => $Params{'ORDERID'},
            'TransTime' => GetCurrentDateTime,
            #'TransStatus' => TRANS_STATUS_NEW
         } );

    # debugging
    my $paymentURL;

    if ($isTestMode) {
        $paymentURL = LoadRootObject()->get($paymentTypeName . '_ServiceTestURL');
    } else {
        $paymentURL = LoadRootObject()->get($paymentTypeName . '_ServiceURL');
    }

    LogPayment(
        'Ogone',
        "params for payment line item to send to url : $paymentURL\n",
        \%Params
    ) ;

    # set TLEs
    $Servlet->vars(
        'OgoneParams',
        [
            map {
                {
                    'Name' => $_,
                    'Value' => $Params{ $_ }
                }
            } keys %Params
        ]
    ) ;

    return;
}

#==============================================================================
sub ViewMultiPaymentOgone {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    $self->ViewPaymentOgone($Servlet);
    return;
}

#==============================================================================
sub PaymentBackToBasket {
   my $self = shift;
    my ($Servlet) = @_;
    $self->BlockMonitor($Servlet);

    my $hFormValues = $Servlet->params();

    my $PaymentLineItemGUID = $hFormValues->{'GUID'};
    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);
    my $BasketOrder = $PaymentLineItem->container->parent;
    my $ObjectID = $BasketOrder->id;

    LogPayment(
        'Ogone',
        "PaymentBackToBasket $PaymentLineItemGUID:\n",
        {
            'OrderId' => $ObjectID
        }
    ) ;

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    $Servlet->vars( 'FormError','1');

    #View action for the confirmation page
    $Servlet->vars( 'ViewAction','PaymentBackToBasket');
    $Servlet->vars( 'ObjectID', $ObjectID);

    return;
}

1;
