#========================================================================================
# package      DE_EPAGES::PriceList::Hooks::Shop
# state        public
# description  Creates or removes pricelists if pricelist feature
#               is activated or removed
#========================================================================================
package DE_EPAGES::PriceList::Hooks::Shop;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::PriceList::API::PriceList qw ( CreateUnityPriceListForCustomerGroup );


#========================================================================================
# function     OnAfterUpdateShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShopFeature( $hShopFeature );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_InsertShopFeature.
#               Creates pricelists for each customergroup if 'Pricelist' feature is enabled.
#               Removes pricelists if 'Pricelist' feature is removed.
#----------------------------------------------------------------------------------------
# input        $hShopFeature | hook parameters
#               <ul>
#                 <li>ShopID     - shop id    - int
#                 <li>FeatureID  - feature id - int
#                 <li>MaxValue   - maximum feature value | int</li>
#               </ul>
#========================================================================================
sub OnAfterUpdateShopFeature {
    my ($hShopFeature) = @_;
    my $ShopID = $hShopFeature->{'ShopID'};
    my $FeatureID = $hShopFeature->{'FeatureID'};
    my $Feature = LoadObject ( $FeatureID );
    if ($Feature->alias eq 'PriceLists') {
        my $Shop = LoadObject( $ShopID );

            # only for Unity shops
            if ($Shop->get('IsUnityRegistered')) {
                my $CustomerGroups = $Shop->get('CustomerGroups');

                if ($hShopFeature->{'MaxValue'}) {
                    #nothing to create if no customer groups exists
                    return unless defined $CustomerGroups;

                    foreach my $Group (@$CustomerGroups) {
                        next if $Group->alias eq 'NewCustomer';

                        CreateUnityPriceListForCustomerGroup($Group);
                    }
                }
            }
        return;
    }
}
1;
