use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script                qw ( RunScript);
use DE_EPAGES::Database::API::Connection        qw ( RunOnStore);
use DE_EPAGES::Object::API::Factory             qw ( LoadObjectByPath );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );


sub Main {

    local $| = 1;

    my $Password;
    my $Help;
    my $StoreName;
    my $ShopName;
    my $Enable;

    GetOptions(
        'help'              => \$Help,
        'passwd=s'          => \$Password,
        'storename=s'       => \$StoreName,
        'shopname=s'        => \$ShopName,
        'enable=s'          => \$Enable,
    );

    Usage() if $Help;
    Usage( "Missing required parameter -storename" ) unless defined $StoreName;
    Usage( "Missing required parameter -enable" ) unless defined $Enable;

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {
            my $aShops;
            if ( $ShopName ) {
              $aShops = [ LoadObjectByPath('/Shops/'.$ShopName) ];
            } else {
              $aShops = LoadObjectByPath('/Shops')->children();
            }

            my $State = $Enable ? 1 : 0;
            my $Enabling = $Enable ? 'Enabling' : 'Disabling';
            print "$Enabling recaptcha: \n";
            foreach my $Shop ( @$aShops ) {
              next if $Shop->alias =~ m/^PreCreated/g;
              $Shop->set({
                'EnableReCaptcha' => $State,
              });
              print " ".$Shop->alias."\n";
            }

            SynchronizeCache();
        }
    );
    printf("need %.3f seconds\n", time()-$time1);
}

sub Usage {
    print "\n@_\n\n" if @_;
    print <<END_USAGE;

Usage:
    perl $0 [options] [flags]

options:
    -passwd                      database user password             (optional)
    -storename                   the name of the Store              (required)
    -shopname                    name of the shop                   ()
    -enable                      turn recaptcha on                  (required)

flags:
    -help       show the command line options

Example:
    perl $0 -storename Store -shopname DemoShop -enable 1

Description:
    Sets the recaptcha status on/off for a/all shop(s)
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);